/*
 *  GUI/EntriesList/ListView.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */

using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using Gtk;
using Gdk;
using Incollector.GUI;
using Incollector.Engine;
using Incollector.Searching;
using Incollector.DataTypes;


namespace Incollector.GUI.Widgets {

	public class ListView : EntriesList {
		
		protected enum ColsEnum {
			Icon     = 0,
			Item     = 1,
			DateText = 2,
			Stars    = 3,
			ID       = 4,
			Category = 5,
			Date     = 6,
			Rating   = 7
		};
		
		protected enum MainNotebookEnum {
			Normal  = 0,
			Message = 1
		};
		
		protected TargetEntry [] DestTable = new TargetEntry [] {
			new TargetEntry ("STRING", 0, 0),
			new TargetEntry ("text/uri-list", 0, 0),
			new TargetEntry ("text/plain", 0, 0),
			new TargetEntry ("_NETSCAPE_URL", 0, 0),
			new TargetEntry ("application/x-rootwin-drop", 0, 0)
		};
		
		protected Gdk.Pixbuf [] Stars = {
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-0.png")),
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-1.png")),
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-2.png")),
				new Gdk.Pixbuf (Assembly.GetEntryAssembly (), ("stars.bigger-3.png"))
			};
		
		
		protected TreeView     list;
		protected Hashtable    CategoriesPixbufs;
		protected Gtk.Notebook MainNotebook;
		protected Gtk.Label    MessageLabel;
		protected bool         ShowingListMenu;
		
		
		public ListView (Gtk.Window ParentWindow, EntryPreview preview, TagsList tags, SearchEntry SearchQuery)
		{
			this.PreviewWidget     = preview;
			this.TagsListWidget    = tags;
			this.SearchEntryWidget = SearchQuery;
			this.ParentWindow      = ParentWindow;
			
			Create ();
		}
		
		
		/// <summary>Number of entries on the list.</summary>
		public override int NEntries {
			get { return (list.Model as ListStore).IterNChildren (); }
		}
		
		
		/// <summary>Number of selected entries on the list.</summary>
		public override int NSelectedEntries {
			get { return GetSelectedEntries ().Count; }
		}
		
		
		/// <summary>Gets list widget as Gtk.Widget</summary>
		public override Widget FocusWidget {
			get { return list as Gtk.Widget; }
		}

		
		
		protected void Create ()
		{
			// Database callbacks
			DB.Loaded  += new DB.LoadHandler   (this.OnDBLoaded);
			DB.Added   += new DB.AddHandler    (this.OnDBAddedEntry);
			DB.Updated += new DB.UpdateHandler (this.OnDBUpdatedEntry);
			DB.Removed += new DB.RemoveHandler (this.OnDBRemovedEntry);
			
			
			// TagsListWidget callbacks
			this.TagsListWidget.TagsActivityChanged += new TagsList.TagActivityChangedHandler (this.OnTGWTagsActivityChanged);
			
			
			// SearchEntryWidget callbacks
			SearchEntryWidget.FindActivated += new SearchEntry.FindActivateHandler (this.OnSEWFindActivated);
			
			
			// SearchFolders
			SearchFolders.Activated += new SearchFolders.ActivatedHandler (OnSFActivated);
			SearchFolders.Modified += new SearchFolders.ModifiedHandler (OnSFModified);
			
			
			// Loading categories pixbufs
			CategoriesPixbufs = new Hashtable ();
			foreach (DataType type in DataTypes.DataTypes.DataTypesList)
				CategoriesPixbufs [type.Name] = type.Icon16;
			
			
			// creating MainNotebook widget
			this.MainNotebook = new Gtk.Notebook ();
			this.MainNotebook.ShowBorder = false;
			this.MainNotebook.ShowTabs = false;
			this.Add (MainNotebook);
			
			
			// creating treeview widget
			this.list = new TreeView ();
			Gtk.ListStore store = new Gtk.ListStore (typeof (Gdk.Pixbuf), typeof (string), typeof (string), typeof (Gdk.Pixbuf), typeof (int), typeof (string), typeof (string), typeof (int));
			
			list.Model = store;
			list.EnableSearch = false;
			list.RulesHint = true;
			list.HeadersVisible = true;
			list.Selection.Mode = SelectionMode.Multiple;
			list.Selection.Changed += new EventHandler (on_selection_changed);
			list.RowActivated += new RowActivatedHandler (on_row_activated);
			list.DragDataReceived += new DragDataReceivedHandler (on_drag_data_received);
			list.DragMotion += new DragMotionHandler (on_drag_motion);
			list.ButtonReleaseEvent += new ButtonReleaseEventHandler (on_button_release_event);
			list.KeyReleaseEvent += new KeyReleaseEventHandler (on_key_release_event);
			Gtk.Drag.DestSet (list, DestDefaults.All, DestTable, DragAction.Copy | DragAction.Move);
			
			
			CellRendererPixbuf IconRenderer = new CellRendererPixbuf ();
			TreeViewColumn IconColumn = new TreeViewColumn ("", IconRenderer, "pixbuf", (int) ColsEnum.Icon);
			list.AppendColumn (IconColumn);
			
			CellRendererText EntryRenderer = new CellRendererText ();
			EntryRenderer.Ellipsize = Pango.EllipsizeMode.End;
			TreeViewColumn EntryColumn = new TreeViewColumn (Catalog.GetString ("Entry"), EntryRenderer, "markup", (int) ColsEnum.Item);
			EntryColumn.Expand = true;
			list.AppendColumn (EntryColumn);
			
			CellRendererText DateRenderer = new CellRendererText ();
			TreeViewColumn DateColumn = new TreeViewColumn (Catalog.GetString ("Date"), DateRenderer, "text", (int) ColsEnum.DateText);
			DateColumn.Expand = false;
			list.AppendColumn (DateColumn);
			
			CellRendererPixbuf RatingRenderer = new CellRendererPixbuf ();
			RatingRenderer.Xalign = 0;
			list.AppendColumn (Catalog.GetString ("Rating"), RatingRenderer, "pixbuf", (int) ColsEnum.Stars);
			
			
			// creating scrollbar widget
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.ShadowType = Gtk.ShadowType.In; 
			scroll.Add (list);
			this.MainNotebook.AppendPage (scroll, null);
			
			
			// creating message widget
			Gtk.Viewport bg = new Gtk.Viewport ();
			Colors.Modify (Colors.WHITE, bg);
			
			this.MessageLabel = new Gtk.Label ();
			this.MessageLabel.WidthRequest = 550;
			this.MessageLabel.Wrap = true;
			this.MessageLabel.Justify = Gtk.Justification.Center;
			this.MessageLabel.Xalign = (float) 0.50;
			this.MessageLabel.UseMarkup = true;
			bg.Add (this.MessageLabel);
			
			this.MainNotebook.AppendPage (bg, null);
			
			
			this.ShowAll ();
		}
		

		protected void OnDBLoaded ()
		{
			TagsListWidget.LoadTagsFromDB ();
			TagsListWidget.LoadTagsActive (new ArrayList (Engine.Settings.Get ("DisabledTags").Split (',')));
			
			this.ReloadEntries ();
		}
		
		
		protected void OnDBAddedEntry (Engine.EntryStruct entry)
		{
			this.TagsListWidget.AddTags (entry.tags);
			ArrayList EnabledTags = this.TagsListWidget.EnabledTagsList ();
			
			if ((EntryCheckSearchFolder (entry)) && (EntryCheckSearch (entry)) &&
					(EntryCheckTags (entry, EnabledTags)))
				this.AddEntry (entry);
			
			EntriesMessage ();
			LoadPreview (true);
		}
		
		
		protected void OnDBUpdatedEntry (Engine.EntryStruct entry, Engine.EntryStruct OldEntry)
		{
			this.TagsListWidget.ApplyDiff (new ArrayList (OldEntry.tags), new ArrayList (entry.tags));
			ArrayList EnabledTags = this.TagsListWidget.EnabledTagsList ();
			
			if ((EntryCheckSearchFolder (entry)) && (EntryCheckSearch (entry)) &&
					(EntryCheckTags (entry, EnabledTags)))
				this.ModifyEntry (entry);
			else
				this.RemoveEntry (entry);
			
			
			EntriesMessage ();
			LoadPreview (true);
		}
		
		
		protected void OnDBRemovedEntry (Engine.EntryStruct entry)
		{
			this.RemoveEntry (entry);
			this.TagsListWidget.RemoveTags (entry.tags);
			
			EntriesMessage ();
			LoadPreview ();
		}
		
		
		protected void OnTGWTagsActivityChanged ()
		{
			this.ReloadEntries ();
		}
		
		
		protected void OnSEWFindActivated (string Query)
		{
			this.ReloadEntries ();
		}
		
		
		protected void OnSFActivated (SearchFolder folder)
		{
			this.ReloadEntries ();
		}
		
		
		protected void OnSFModified (SearchFolder folder)
		{
			if (SearchFolders.ActiveFolder.ID == folder.ID)
				this.ReloadEntries ();
		}
		
		
		/// <summary>Shows entries message.</summary>
		protected void ShowEntriesMessage (string title, string text)
		{
			this.MessageLabel.Markup = "<span size='x-large' weight='bold'>" + title + "</span>\n\n" + text;
			this.MainNotebook.Page = (int) MainNotebookEnum.Message;
		}
		
		
		/// <summary>Hides entries message.</summary>
		protected void HideEntriesMessage ()
		{
			this.MainNotebook.Page = (int) MainNotebookEnum.Normal;
		}
		
		
		protected void EntriesMessage ()
		{
			if (this.NEntries > 0)
				HideEntriesMessage ();
			else {
				if (DB.NEntries > 0) {
					if (this.SearchEntryWidget.Searching) // if showing search results
						ShowEntriesMessage (Catalog.GetString ("No results were found!"), "");
					else
						ShowEntriesMessage (Catalog.GetString ("Entries couldn't be showed!"),
											Catalog.GetString ("Change browsing settings to see entries."));
				} else
					ShowEntriesMessage (Catalog.GetString ("There are no entries!"),
										Catalog.GetString ("To add a new entry click <u>Add entry</u> button."));
			}
		}
		
		
		public override void ReloadEntries ()
		{
			this.ShowEntriesMessage (Catalog.GetString ("Loading entries..."), "");
			Tools.Repaint ();
			this.Clear ();
			
			ArrayList EnabledTags;
			EnabledTags = TagsListWidget.EnabledTagsList ();
			
			foreach (Engine.EntryStruct entry in DB.Data)
				if (EntryCheckSearchFolder (entry) && EntryCheckSearch (entry) &&
							EntryCheckTags (entry, EnabledTags))
					this.AddEntry (entry);
			
			EntriesMessage ();
			this.HideEntriesMessage ();
		}

		
		
		/* Can entry be showed ? (checks search folders) */
		private bool EntryCheckSearchFolder (Engine.EntryStruct ent)
		{
			return SearchFolders.ActiveFolder.CheckEntry (new Engine.Entry (ent));
		}
		
		
		/* Can entry be showed ? (search) */
		private bool EntryCheckSearch (Engine.EntryStruct ent)
		{
			if (this.SearchEntryWidget.Query != "")
				return ent.data.Search (this.SearchEntryWidget.Query) ||
					   Engine.Text.Search (this.SearchEntryWidget.Query, ent.comment) ||
					   Engine.Text.Search(this.SearchEntryWidget.Query, ent.GetTagsString(false));
			
			return true;
		}
		
		
		/* Can entry be showed ? (checks what tags can entries be showed with) */
		private bool EntryCheckTags (Engine.EntryStruct entry, ArrayList EnabledTags)
		{
			if (entry.tags.Count == 0)
				return true;
			
			foreach (string tag in entry.tags)
				foreach (string stag in EnabledTags)
					if (stag == tag)
						return true;
			
			return false;
		}

		
		protected void on_selection_changed (object o, EventArgs args)
		{
			SelectionChanged (list.Selection.CountSelectedRows ());
		}
		
		
		protected void on_row_activated (object o, RowActivatedArgs args)
		{
			this.ModifySelectedEntry ();
		}
		
		
		protected void on_button_release_event (object o, ButtonReleaseEventArgs args)
		{
			if ((args.Event.Button == 3) && (list.Selection.CountSelectedRows () == 1)) {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				Glade.XML     m     = Glade.XML.FromAssembly ("gui.glade", "EntriesListMenu", null);
				
				m.Autoconnect (this);
				Gtk.Menu menu = (Gtk.Menu) m.GetWidget ("EntriesListMenu");
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				ShowingListMenu = true;
				
				Engine.EntryStruct ent = DB.GetEntryByID ((int) store.GetValue (itr, (int) ColsEnum.ID));
				switch (ent.rating) {
					case 0: (m.GetWidget ("no_rating") as Gtk.RadioMenuItem).Active = true; break;
					case 1: (m.GetWidget ("rating_1") as Gtk.RadioMenuItem).Active  = true; break;
					case 2: (m.GetWidget ("rating_2") as Gtk.RadioMenuItem).Active  = true; break;
					case 3: (m.GetWidget ("rating_3") as Gtk.RadioMenuItem).Active  = true; break;
				}
				
				ShowingListMenu = false;
				
				menu.Show ();
				menu.Popup ();
			
			}
		}
		
		
		protected void on_key_release_event (object o, Gtk.KeyReleaseEventArgs args)
		{
			if (args.Event.Key == Gdk.Key.Escape) {
				if (this.SearchEntryWidget != null)
					this.SearchEntryWidget.Escape ();
			}
		}
		
		
		protected void on_add_activate (object o, EventArgs args)
		{
			AddEntry ("", "");
		}
		
		
		protected void on_remove_activate (object o, EventArgs args)
		{
			RemoveSelectedEntries ();
		}
		
		
		protected void on_remove_entries_activate (object o, EventArgs args)
		{
			RemoveSelectedEntries ();
		}
		
		
		protected void on_edit_entry_activate (object o, EventArgs args)
		{
			this.ModifySelectedEntry ();
		}
		
		
		protected void on_no_rating_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 0;
				ent.Save ();
				
			}
		}
		
		
		protected void on_rating_1_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 1;
				ent.Save ();
			}
		}
		
		
		protected void on_rating_2_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 2;
				ent.Save ();
			}
		}
		
		
		protected void on_rating_3_activate (object o, EventArgs args)
		{
			if ((ShowingListMenu) || ((o as RadioMenuItem).Active == false))
				return;
			
			if (list.Selection.CountSelectedRows () == 1) {
				Gtk.TreeIter  itr;
				Gtk.ListStore store = list.Model as ListStore;
				
				store.GetIter (out itr, list.Selection.GetSelectedRows () [0]);
				
				Engine.Entry ent = new Engine.Entry ((int) store.GetValue (itr, (int) ColsEnum.ID));
				ent.Rating = 3;
				ent.Save ();
			}
		}
		
		
		protected void on_drag_data_received (object o, DragDataReceivedArgs args)
		{
			if (args.SelectionData.Text == null)
				return;
			
			if (args.SelectionData.Text.StartsWith ("incollector:tag")) { // if tag
				int EntryID = this.GetDroppedEntryID (args);
				if (EntryID == -1)
					return;
				
				Engine.Entry entry = new Engine.Entry (EntryID);
				string       tag   = this.TagsListWidget.GetDraggingTag (args);
				
				if (entry.TagExists (tag) == false) {
					entry.AddTag (tag);
					entry.Save ();
				}
			} else {
				string url = new System.Text.StringBuilder (System.Text.Encoding.UTF8.GetString (args.SelectionData.Data)).ToString ();
				
				try {
					Uri addr = new Uri (url);
					
					url = url.Replace ("file:///", "").Trim ();
					
					if (! Engine.FilesPaths.IsWinSystem ())
						url = "/" + url;
					else {
						url = url.Replace (Convert.ToChar (13), '*');
						url = url.Replace (Convert.ToChar (10), '*');
						url = url.Replace ('/', '\\');
						url = url.Replace ("*", "");
					}
				
					if (addr.IsFile == false)
						if ((addr.ToString ().StartsWith ("http")) || (addr.ToString ().StartsWith ("ftp")))
							this.AddEntry (addr.ToString (), "webaddress");
				} catch {
					return;
				}
			}
		}
		
		
		protected void on_drag_motion (object o, DragMotionArgs args)
		{
			TreePath path;
			TreeViewDropPosition pos;
			
			if (list.GetDestRowAtPos (args.X, args.Y, out path, out pos))
				list.SetDragDestRow (path, TreeViewDropPosition.IntoOrAfter);
		}
		
		
		/// <summary>Clears entries list.</summary>
		public override void Clear ()
		{
			(list.Model as ListStore).Clear ();
		}
		
		
		public override void LoadPreview (bool ForceLoad)
		{
			if (PreviewWidget == null)
				return;
			
			if (this.NSelectedEntries == 1) {
				int EID = (int) this.GetSelectedEntries () [0];
				if ((PreviewWidget.EntryID == EID) && (ForceLoad == false))
					return;
				
				PreviewWidget.LoadPreview (new Engine.Entry (EID));
			} else
				PreviewWidget.LoadPreview (new Engine.Entry (-1));
		}
		
		
		/// <summary>Returns selected entries id numbers.</summary>
		public override ArrayList GetSelectedEntries ()
		{
			ArrayList       AllItems = new ArrayList ();
			Gtk.ListStore   store    = list.Model as Gtk.ListStore;
			Gtk.TreeIter    itr;
			
			foreach (Gtk.TreePath pth in list.Selection.GetSelectedRows ()) {
				store.GetIter (out itr, pth);
				AllItems.Add (store.GetValue (itr, (int) ColsEnum.ID));
			}
			
			return AllItems;
		}
		
		
		public override int GetDroppedEntryID (Gtk.DragDataReceivedArgs args)
		{
			Gtk.TreePath path;
			Gtk.TreeViewDropPosition pos;
			Gtk.TreeIter iter;
			
			list.GetDestRowAtPos (args.X, args.Y, out path, out pos);
			(list.Model as ListStore).GetIter (out iter, path);
			
			return (int) (list.Model as ListStore).GetValue (iter, (int) ColsEnum.ID);
		}
		
		
		public override void AddEntry (string Content, string DataTypeName)
		{
			this.SetParentSensitive (false);
			
			EntryDialog dlg;
			
			if (TagsListWidget != null)
				dlg = new EntryDialog (this.ParentWindow, this.TagsListWidget.Tags, DataTypeName, Content);
			else
				dlg = new EntryDialog (this.ParentWindow, new string [] {}, DataTypeName, Content);
			
			dlg.dialog.Show ();
			if (dlg.dialog.Run () == (int) ResponseType.Ok)
				dlg.GetEntry ().Save ();
			
			dlg.dialog.Destroy ();
			
			this.SetParentSensitive (true);
		}
		
		
		public override void RemoveSelectedEntries ()
		{
			ArrayList selected = this.GetSelectedEntries ();
			
			if (selected.Count >= 1) {
				Gtk.MessageDialog msg;
				if (selected.Count == 1)
					msg = new MessageDialog (this.ParentWindow, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, Catalog.GetString ("Are you really sure you want to remove selected entry?"));
				else
					msg = new MessageDialog (this.ParentWindow, DialogFlags.DestroyWithParent, MessageType.Question,
						ButtonsType.YesNo, Catalog.GetString ("Are you really sure you want to remove selected entries?"));
				
				this.SetParentSensitive (false);
				msg.Show ();
				if (ResponseType.Yes == (ResponseType) msg.Run ())
					foreach (int EntryID in selected)
						(new Engine.Entry (EntryID)).Remove ();
				
				this.SetParentSensitive (true);
				msg.Destroy ();
			}
		}
		
		
		public override void ModifySelectedEntry ()
		{
			if (list.Selection.CountSelectedRows () != 1)
				return;
			
			Gtk.TreeIter itr;
			(list.Model as ListStore).GetIter (out itr, list.Selection.GetSelectedRows () [0]);


			Engine.Entry entry = new Engine.Entry ((int) (list.Model as ListStore).GetValue (itr, (int) ColsEnum.ID));
			
			this.SetParentSensitive (false);
			
			EntryDialog dlg;
			
			if (TagsListWidget != null)
				dlg = new EntryDialog (this.ParentWindow, this.TagsListWidget.Tags, entry);
			else
				dlg = new EntryDialog (this.ParentWindow, new string [] {}, entry);
			
			dlg.dialog.Show ();
			if (dlg.dialog.Run () == (int) ResponseType.Ok)
				dlg.GetEntry ().Save ();
			
			dlg.dialog.Destroy ();
			this.SetParentSensitive (true);
		}
		
		
		/// <summary>Append a new entry.</summary>
		protected void AddEntry (Engine.EntryStruct entry)
		{
			Gtk.ListStore store = this.list.Model as ListStore;
			Gtk.TreeIter itr = (list.Model as ListStore).Prepend ();
 			
 			store.SetValue (itr, (int) ColsEnum.Icon, CategoriesPixbufs [entry.data.Name]);
 			store.SetValue (itr, (int) ColsEnum.Item, Engine.Text.PrepareText (Engine.Text.Crop (entry.data.EntryTitle), true));
 			store.SetValue (itr, (int) ColsEnum.DateText, entry.GetDate (Engine.Settings.GetBool ("DateFirstYear")));
 			store.SetValue (itr, (int) ColsEnum.Stars, this.Stars [entry.rating]);
			store.SetValue (itr, (int) ColsEnum.ID, entry.id);
			store.SetValue (itr, (int) ColsEnum.Category, entry.data.Title);
			store.SetValue (itr, (int) ColsEnum.Date, entry.date.ToString ("yyyy-MM-dd"));
			store.SetValue (itr, (int) ColsEnum.Rating, entry.rating);
			
			if (list.IsRealized) {
				while (Application.EventsPending ()) {
					Application.RunIteration();
				}
				
				list.ScrollToPoint(0, 0);
			}
		}
		
		
		/// <summary>Gets entry iter by specified id.</summary>
		private Gtk.TreeIter GetIterByEntryID (int EntryID)
		{
			Gtk.ListStore store = list.Model as Gtk.ListStore;
			Gtk.TreeIter  itr;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if ((int) store.GetValue (itr, (int) ColsEnum.ID) == EntryID)
					return itr;
			
			return Gtk.TreeIter.Zero;
		}
		
		
		/// <summary>Modifies existing entry.</summary> 
		protected void ModifyEntry (Engine.EntryStruct NewEntry)
		{
			Gtk.ListStore store = this.list.Model as ListStore;
			
			Gtk.TreeIter itr = GetIterByEntryID (NewEntry.id);
			if (store.IterIsValid (itr) == false)
				return;
			
			string item = Engine.Text.PrepareText (Engine.Text.Crop (NewEntry.data.EntryTitle), true);
			store.SetValue (itr, (int) ColsEnum.Item, item);
			
			store.SetValue (itr, (int) ColsEnum.Stars, this.Stars [NewEntry.rating]);
			store.SetValue (itr, (int) ColsEnum.ID, NewEntry.id);
			store.SetValue (itr, (int) ColsEnum.DateText, NewEntry.GetDate (Engine.Settings.GetBool ("DateFirstYear")));
			store.SetValue (itr, (int) ColsEnum.Category, NewEntry.data.Title);
			store.SetValue (itr, (int) ColsEnum.Date, NewEntry.date.ToString ());
			store.SetValue (itr, (int) ColsEnum.Rating, NewEntry.rating);
		}
		
		
		/// <summary>Removes selected entry.</summary>
		protected void RemoveEntry (Engine.EntryStruct entry)
		{
			Gtk.TreeIter itr = GetIterByEntryID (entry.id);
			if ((list.Model as ListStore).IterIsValid (itr) == false)
				return;
			
			(list.Model as ListStore).Remove (ref itr);
		}
		
		
		protected bool Exists (int EntryID)
		{
			Gtk.ListStore store = list.Model as Gtk.ListStore;
			Gtk.TreeIter  itr;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (EntryID == (int) store.GetValue (itr, (int) ColsEnum.ID))
					return true;
			
			return false;
		}
		
		
		protected void SetParentSensitive (bool Sensitive)
		{
			this.ParentWindow.Sensitive = Sensitive;
		}
	}
}
